//================================================================================================
//  THEREMINO SCRIPT
//================================================================================================
// Theremino Script semplifica al massimo la gestione di semplici operazioni
// del tipo "accendo un led", "attendo un secondo", "spengo il led", "ripeto tre volte"
// Le complessita' di DotNet vengono nascoste per quanto possibile.
//------------------------------------------------------------------------------------------------
// L'approccio semplificato e' valido solo per semplici programmi di TEST.
// Nel caso che il progetto si allunghi oltre qualche pagina si consiglia di usare Visual Studio
//  di Microsoft che e' il fratello maggiore di questo programma.
//------------------------------------------------------------------------------------------------
// La versione Express di Visual Studio e' gratuita e funziona molto meglio di questo rozzo SCRIPT
//------------------------------------------------------------------------------------------------
// E' bene sapere che ad ogni RUN il programma viene salvato automaticamente.
// Se qualcosa va storto, salvare, chiudere e riaprire.
//================================================================================================


//================================================================================================
//  Example 1
//=================================================================================================
// Questo esempio mostra come accedere agli slot del sistema anche con il linguaggio CSharp 
// Con il sistema theremino si usa normalmente il VbNet ma per chi è abituato ad Arduino
// il linguaggio CSharp é molto più comodo (quasi identico al C di Arduino)
//=================================================================================================

//=================================================================================================
// Variabili e costanti
// Qui si dichiarano le variabili e le costanti che deveono essere in comune a tutte le funzioni
//=================================================================================================
//const int K1 = 12345;
//static bool BooleanVar1 = false;
//static int IntVar1 = 12345;
//static float FloatVar1 = 123.45F; // la "F" serve per impostare un valore letterale di tipo Float


//-------------------------------------------------------------------------------------------------
// Nella funzione "Main_Thread" si puo' eseguire ogni genere di operazione
// Si possono fare PAUSE lunghe a piacere, cicli LOOP, FOR, WHILE etc...
// Però non è bene aggiornare i controlli della finestra in questa funzione, 
// se necessario si impostano variabili comuni che verranno poi visualizzate dalla Timed_Update 
//-------------------------------------------------------------------------------------------------
// Se si arriva alla fine della funzione allora la funzione ricomincia immediatamente da capo
//-------------------------------------------------------------------------------------------------
static void Main_Thread()
{
	// Esempio che divide il valore dello Slot1 in quattro parti e imposta gli slot 2, 3, 4 e 5
	// Utilizzare Theremino SlotViewer per vedere i risultati
	// Non è necessario collegare un Master, molte cose si fanno anche senza hardware,
	// si possono pilotare applicazioni che suonano o leggere valori da una WebCam
	// L'hardware serve solo qando si deve uscire o entrare dal PC attraverso la USB
	
	float n = ReadSlot(1);
	
	switch ((int)n / 250)
	{
		case 0:
			WriteSlot(2, 1000);	
			WriteSlot(3, 0);
			WriteSlot(4, 0);
			WriteSlot(5, 0);
			break;
		case 1:
			WriteSlot(2, 0);	
			WriteSlot(3, 1000);
			WriteSlot(4, 0);
			WriteSlot(5, 0);
			break;
		case 2:
			WriteSlot(2, 0);	
			WriteSlot(3, 0);
			WriteSlot(4, 1000);
			WriteSlot(5, 0);
			break;
		case 3:
			WriteSlot(2, 0);	
			WriteSlot(3, 0);
			WriteSlot(4, 0);
			WriteSlot(5, 1000);
			break;
	}
}

// ------------------------------------------------------------------------------------------------
//  Questa funzione viene chiamata 20 volte al secondo
//  L'aggiornamento dei controlli che vanno rinfrescati regolarmente deve avvenire qui
//  Ma niente PAUSE e niente LOOP, solo esecuzioni rapide
//  Aggiornare i valori (ad esempio i "Text" delle textbox) e poi uscire subito dalla funzione
// ------------------------------------------------------------------------------------------------
static void Timed_Update()
{
	// Notare che l'aggiornamento della label è stato aggiunto qui al posto che in Main_Thread
	// Qui avviene 20 volte al secondo (la velocità giusta per le comunicazioni con l'utente)
	// Invece nella Main_Thread avverrebbe 1000 volte al secondo e caricherebbe molto la CPU
	// Oppure avverrebbe con molto ritardo se nella Main_Thread ci fossero pause o loops
	// Non ha importanza che così facendo si deve rileggere lo slot, ci vuole meno di un microsecondo
	Label1.Text = "Slot 1 = " + ReadSlot(1).ToString();										
}

// ------------------------------------------------------------------------------------------------
//  Le seguenti funzioni sono "Eventi" - Vengono chiamate al verificarsi degli eventi relativi
//  Abilitando i beep si può "sentire" quando gli eventi accadono
// ------------------------------------------------------------------------------------------------
static void Window_Opening()
{
	//Beep();
}

static void Window_Closing()
{
	//Beep();
}

static void TrackBar1_ValueChanged(Object myObject, EventArgs myEventArgs)
{
	//Beep();
	WriteSlot(1, TrackBar1.Value);
}

// ------------------------------------------------------------------------------------------------
// Componenti della interfaccia utente
// Qui si possono aggiungere controlli e modificare le loro proprietà
// Per aggiungere un nuovo controllo copiarne uno esistente
// Ricordarsi di cambiare nome ai nuovi controlli (Label2, Label3 etc..) 
// Ricordarsi di cambiare posizione ai nuovi controlli (altrimenti sono sopra ai precedenti)
// Per alcuni controlli è necessario avere un evento per cui si deve anche aggiungere la funzione
// (e incrementarne il nome dell'evento: TrackBar2_ValueChanged, TrackBar3_ValueChanged..)
// ------------------------------------------------------------------------------------------------
static Label Label1 = new Label();
static TrackBar TrackBar1 = new TrackBar();

static void InitializeComponents()
{
	// ------------------------------------------------------------------ Form (cambiare il titolo e la posizione)
	Form1.Text = "Theremino Script - Example 1";
	Form1.ClientSize = new Size(640, 320);
	Form1.MinimumSize = Form1.Size;
	Form1.StartPosition = FormStartPosition.CenterScreen;
	// ------------------------------------------------------------------ Label 1
	Label1.BorderStyle = BorderStyle.Fixed3D;
	Label1.Location = new Point(10, 10);
	Label1.Font = new Font("Arial", 12 );
	Label1.Size = new Size(620, 21);
	Label1.Anchor = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
	Form1.Controls.Add(Label1);
	// ------------------------------------------------------------------ TrackBar 1
	TrackBar1.Location = new Point(10, 60);
	TrackBar1.Size = new Size(45, 149);
	TrackBar1.Orientation = Orientation.Vertical;
	TrackBar1.TickStyle = TickStyle.Both;
	TrackBar1.Maximum = 1000;
	TrackBar1.TickFrequency = 100;
	TrackBar1.LargeChange = 10;
	Form1.Controls.Add(TrackBar1);
	TrackBar1.ValueChanged += new EventHandler(TrackBar1_ValueChanged);
}

// ------------------------------------------------------------------------------------------------
//  Funzioni di utilità semplificate
// ------------------------------------------------------------------------------------------------
static void PausaMS(Int32 mS)
{
	System.Threading.Thread.Sleep(mS);
}
static void Beep()
{
	System.Media.SystemSounds.Beep.Play();
}



// ===============================================================================================
//  LA GESTIONE SEMPLIFICATA FINISCE QUI
//  MODIFICARE LA PARTE SEGUENTE CON ATTENZIONE
// ===============================================================================================
static Form Form1;
static void Main(String[] pars)
{
	if (Form1 != null) return;
	Form1 = new Form();
	Form1.Closing += new CancelEventHandler(Form1_Closing);
	Application.EnableVisualStyles();
	Form1.SuspendLayout();
	InitializeComponents();
	Form1.ResumeLayout();
	Form1.PerformLayout();
	StartMainTimer();
	StartMainThread();
	Window_Opening();
	Form1.Show();
	while(Form1.Visible)
	{
		Application.DoEvents();
		System.Threading.Thread.Sleep(1);
	}
}
static void Form1_Closing(Object myObject, EventArgs myEventArgs)
{
	MainTimer.Stop();
	MainThread.Abort();
	Window_Closing();
}
// ---------------------------------------------------------------------- Main Thread
static System.Threading.Thread MainThread;
static void StartMainThread()
{
	MainThread = new System.Threading.Thread(ExecuteThread);
	MainThread.IsBackground = true;
	MainThread.Priority = System.Threading.ThreadPriority.AboveNormal;
	MainThread.Start();
}
static void ExecuteThread()
{
	while (true)
	{
		Main_Thread();
		System.Threading.Thread.Sleep(1);
	}
}
// ---------------------------------------------------------------------- Main Timer
static Timer MainTimer;
static void StartMainTimer()
{
	MainTimer = new Timer();
	MainTimer.Tick += new EventHandler(Timer1_Tick);
	MainTimer.Interval = 50;
	MainTimer.Start();
}
static void Timer1_Tick(Object myObject, EventArgs myEventArgs)
{
	Timed_Update();
}


