' ================================================================================================
' - ENGLISH - TEMPERATURE METER
' ================================================================================================
'  This script reads a temperature sensor and shows the temperature with a large display.
'  Implemented sensors are LM35 (3 Euro) and TSIC501 (8 Euro)
'  Sensors are factory trimmed with great precision, LM35 (0.25 degree), TSIC501 (0.1 degree)
'  If required they can be trimmed to +/-0.001 degree with the constant "TempMeterTrim"
' ------------------------------------------------------------------------------------------------

' ================================================================================================
' - ITALIANO - MISURATORE DI TEMPERATURA
' ================================================================================================
'  Questo script legge un sensore di temperatura e presenta la temperatura su un grande display.
'  I sensori implementati sono lo LM35 (3 Euro) e lo TSIC501 (8 Euro)
'  Questi sensori sono calibrati in fabbrica con precisione LM35 (0.25 gradi), TSIC501 (0.1 gradi)
'  Se necessario si possono regolare a +/-0.001 gradi con la costante "TempMeterTrim"
' ------------------------------------------------------------------------------------------------

Const TempMeterSlot As Int32 = 1
Const TempMeterType As String = "LM35" ' implemented "LM35" and "TSIC501"
Const TempMeterTrim As Single = 0.000  ' sensor trimming to 0.001 deg. (pos or neg value)

Sub Main(ByVal s() As String)
	if Form1 IsNot Nothing then return
	Form1 = New Form
	InitializeComponents
	MemoryMappedFile_Init
	While Form1.Visible
		Application.DoEvents
		Threading.Thread.Sleep(10)
	End While
End Sub

Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Form1.Closing
	Timer1.Stop
End Sub

Sub Timer1_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles Timer1.Tick
	static Temp as single =0
	dim NewTemp as single
	select case TempMeterType
		Case "LM35"
			NewTemp = ReadSlot(TempMeterSlot) / 3
		Case "TSIC501"
			NewTemp = ReadSlot(TempMeterSlot) / 333 * (60 - 10) - 10
	end select
	NewTemp += TempMeterTrim
	
	SmoothValue_Pow_Adaptive(Temp, NewTemp, 10)
		
	Label1.Text = Temp.ToString("0.000")
End Sub


' ================================================================================================
'   An adaptive, HiQuality, smoothing function
' ================================================================================================
Private Sub SmoothValue_Pow_Adaptive(ByRef value As Single, ByVal new_value As Single, ByVal speed As Single)
	Dim delta As Single = new_value - value
	Dim delta_sgn As Single = Math.Sign(delta)
	Dim delta_abs As Single = Math.Abs(delta)
	Dim delta_pow As Single = CSng(delta_abs ^ 2) * speed
	if value <> 0 then delta_pow /= abs(value)
	If delta_pow >= delta_abs Then
		value = new_value
	Else
		value += delta_sgn * delta_pow
	End If
End Sub

' ================================================================================================
'   User Interface Components
' ================================================================================================
WithEvents Form1 As Form
WithEvents Timer1 As Timer = New Timer
Dim Label1 As Label = new Label
Dim WithEvents ButtonReset As System.Windows.Forms.Button = New System.Windows.Forms.Button

Sub InitializeComponents()
	Application.EnableVisualStyles
	form1.SuspendLayout
	' ------------------------------------------------------------------ Form
	Form1.Text = "Theremino Script - Temp Meter"
	Form1.FormBorderStyle = FormBorderStyle.Fixed3D
	Form1.MaximizeBox = false
	Form1.ClientSize = New System.Drawing.Size(320, 90)
	Form1.MinimumSize = Form1.Size
	Form1.StartPosition = FormStartPosition.CenterScreen
	' ------------------------------------------------------------------ Label 1
	'Label1.BorderStyle = BorderStyle.Fixed3D
	Label1.Location = New Point(0, 0)
	Label1.Size = Form1.ClientSize
	Label1.BackColor = Color.DarkBlue
	Label1.ForeColor = Color.Gold
	Label1.Font = New Font("Tahoma", 55, FontStyle.Bold)
	Label1.TextAlign = ContentAlignment.MiddleRight
	Label1.Anchor = AnchorStyles.Top or AnchorStyles.Left or _
					AnchorStyles.Right or AnchorStyles.Bottom
	Form1.Controls.Add(Label1)
	' ------------------------------------------------------------------ Timer 1
	Timer1.Interval = 15
	Timer1.Start
	' ------------------------------------------------------------------ 
	Form1.ResumeLayout
	Form1.PerformLayout()
	Form1.Show
End Sub
